/*
* $Id: RCcam.c, v 1.0 2003/03/02
*
* Danny H. Newport
* 3-2-03
* R/C servo camera control
* onboard mounted camera with separate control
*
*      Port layout:
*
*      PORTD     0-4,6 not used
*      PORTD     5  start timer function on processor or restart after timeout
*
*      PORTB     0      camera shutter control servo
*      PORTB     1      led control indicator
       PORTB     2-7    unused PWM outputs

*
*
*   This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.       
* 
*/

#include <io.h>
#include <interrupt.h>
#include <sig-avr.h>
#include <timer.h>
#define __AVR_AT90S2313__

/* timer 0 ini tial count */
/* 0 -> 25,6 us PWM resolution @ 10 Mhz */
/* 56 -> 20 us */
#define STARTCNT0 56
#define CPU_freq   10000000                     //cpu clock frequency
#define CYCLES_PER_US ((CPU_freq+400000)/1000000) 	// cpu cycles per microsecond
#define MAX_PIC     475       //max pictures with 128mb SD chip
#undef ECHO

/* global variables for robot functions */
static uint16_t loop=0;

/* global variables for delay functions */
uint16_t delay_count=CPU_freq/4000;             // used in delay function

//static uint8_t dloop = 0;                     // used for delay loops greater than 255mS

/* global variables used by PWM outputs */
volatile uint8_t pwm[8];
static uint8_t pwm_clk = 0;
static uint8_t pwm_cnt = 0;
volatile uint8_t pwm_config;

/* global variables for serial port */
//static uint8_t recv_bytes = 0;
//static char buffer[10];
volatile uint8_t ready = 0;

/* prototypes */
void main_loop (void);
void delay_us(unsigned short us);
void delay_ms(uint8_t ms);
void delay_sec(uint16_t sec);
void takepic(void);


/*
* Set default values
*/
void init(void)
{
	/* all outputs = pwm */
	pwm_config = 0xFF;
	/* all outputs low */
	//memset (pwm, 0x00, 8);
	
}

/*
* uart transmit complete isr
*/
INTERRUPT (SIG_UART_TRANS)
{
	ready = 1;
}

/*
* Timer isr
*
* Takes 95 cycles (9,5us @10Mhz) with gcc 3.0.3 and -Os optimization
*/
SIGNAL (SIG_OVERFLOW0)
{
	uint8_t tpwm;
	
	/* reload counter, +23 to compensate for all 
	instructions executed between the timer
	overflow and this reload */
	outp (STARTCNT0+23, TCNT0);	
	tpwm = 0;
	if (pwm_clk < pwm[0])
		tpwm |= 1;
	if (pwm_clk < pwm[1])
		tpwm |= 2;
	if (pwm_clk < pwm[2])
		tpwm |= 4;
	if (pwm_clk < pwm[3])
		tpwm |= 8;
	if (pwm_clk < pwm[4])
		tpwm |= 16;
	if (pwm_clk < pwm[5])
		tpwm |= 32;
	if (pwm_clk < pwm[6])
		tpwm |= 64;
	if (pwm_clk < pwm[7])
		tpwm |= 128;
	
	if (pwm_cnt & 1) {		/* in servo mode, only output 1 pulse / 2 */
		tpwm &= pwm_config;
		__asm__ __volatile__ ("nop"); /* improves timing */
	} else {
		__asm__ __volatile__ ("nop");
	}
	
	outp (tpwm, PORTB);		/* set PWM outputs */
	
	pwm_clk++;
	if (pwm_clk == 255)
	{
		pwm_clk = 0;		/* 255 steps not 256 */
		pwm_cnt++;
	}
}

/*
* Convert an ASCII char to a 4 bits value
*/
uint8_t quartet (char c)
{
	/* to upper case */
	if (c >= 'a')
		c = c - 32;
	if (c < 'A')
		return (c - '0');
	else
		return (c - 'A' + 10);
}

char tobcd (uint8_t i)
{
	if (i > 9)
		return (i + 'A' - 10);
	else
		return (i + '0');
}

/*
*  wait for serial buffer to clear
*/
void pause (void)
{
	while (!ready);
	ready = 0;
}


/*
* 
*/
int
main (void)
{
	init();
	/* 10Mhz / 200 = 1 overflow every 20,0 us @ 10MHz */
	/* 255 steps = 5.1 ms */
	outp (0xff, DDRB);		/* portb = output */
	outp (0x00, DDRD);		/* portd = input */
	
	outp ((1 << TOIE0), TIMSK);
	outp (STARTCNT0, TCNT0);	/* timer start val */
	outp (CK, TCCR0);		/* prescale = 1 */
	
	outp ((1 << TXCIE) | (1 << RXEN) | (1 << TXEN), UCR);
	outp (64, UBRR);		/* baud rate =9600 */
	
	sei ();			/* enable interrupts */
	main_loop ();
	return 0;
}

/*
 *  function for endless loop
 */
void main_loop (void)
{
     for(;;)//loop forever
     {
	pwm[0]=40;//position servo
	delay_ms(2);
	pwm[0]=0;//turn off servo
     pwm[1]=254;//turn led off
	loop=0;
     loop_until_bit_is_clear(PIND,PIND5);
	outp(0x00,DDRD);
	int t=0;
	pwm[0]=45;//center servo
	pwm[1]=128;//dim led on
     delay_sec(2);//allow servo to center
     pwm[0]=0;//turn off servo to conserve power
	
	/*for(t=0;t<3;t++)//take 3 pics to keep camera on until airborn - not needed for mem card
	{
          for(loop=0;loop<20;loop++)
		   delay_sec(200);
		takepic();
	}*/
		//loop forever
	delay_sec(50);
     outp(0x00,DDRD);
     for(t=0;t<(MAX_PIC);t++)//set for picture max-3 if using first "takeoff" delay
	{
        // for(loop=0;loop<6;loop++)//delay 18seconds
	    delay_ms(200);
	    takepic();

	}
     pwm[0]=35;//retract servo farther
     delay_ms(2);//give it some time
     pwm[0]=0;//turn off servo
     pwm[1]=254;//turn off led
     
     loop_until_bit_is_clear(PIND,PIND5);
     }
}


/*void delay_ms(uint8_t ms)
{
	int c;
	for(c = 0; c < (CPU_freq / 40000); c++)
	{
	}
}*/

// microsecond delay function
void delay_us(unsigned short us)
{
    unsigned short  delay_loops;
    register unsigned short  i;

    delay_loops = (us+3)/5*CYCLES_PER_US; // +3 for rounding up (dirty)

	// one loop takes 5 cpu cycles
    for (i=0; i < delay_loops; i++) {};
}
// millisecond delay function
void delay_ms(uint8_t ms)
{
        uint16_t cnt; asm volatile (

        "\n" "L_dl1%=:" "\n\t" "mov %A0, %A2" "\n\t" "mov %B0, %B2" "\n" "L_dl2%=:" "\n\t"

        "sbiw %A0, 1" "\n\t" "brne L_dl2%=" "\n\t" "dec %1" "\n\t" "brne L_dl1%=" "\n\t" : "=&w" (cnt) : "r" (ms), "r" (delay_count) );
}

void delay_sec(uint16_t sec)
{
	uint16_t loop=0;
	for(loop=0;loop<10;loop++)
		delay_ms(100);
}

void takepic(void)
{
	
			pwm[0]=63;//take picture - changed 65 to 63
			delay_ms(200);//wait for shutter
			pwm[0]=55;//retract arm
			delay_sec(2);//allow arm time to retract
			pwm[0]=0;//turn off servo until needed

}
